"""sc-githooks - Checks on files committed to Git

Copyright (c) 2021 Scott Lau
Portions Copyright (c) 2021 InnoGames GmbH
Portions Copyright (c) 2021 Emre Hasegeli
"""

from githooks import config
from githooks.base_check import BaseCheck, Severity
from githooks.git import CommittedFile


class CommittedFileCheck(BaseCheck):
    """Parent class for checks on a single committed file

    To check the files, we have to clone ourselves when we are being prepared
    for the CommittedFile.  The subclasses has additional logic on those
    to filter out themselves for some cases.
    """
    committed_file = None

    def prepare(self, obj):
        new = super(CommittedFileCheck, self).prepare(obj)
        if not new or not isinstance(obj, CommittedFile):
            return new

        new = new.clone()
        new.committed_file = obj
        return new

    def __str__(self):
        return '{} on {}'.format(type(self).__name__, self.committed_file)


class CommittedFileSizeCheck(CommittedFileCheck):
    """Special check for committed file size"""

    def get_problems(self):
        if self.committed_file.get_file_size() >= config.COMMIT_FILE_MAX_SIZE:
            yield (
                Severity.ERROR,
                '提交 {} 的文件 {} 大小超过 {}, 即 {} MB'
                    .format(self.committed_file.commit, self.committed_file.path,
                            config.COMMIT_FILE_MAX_SIZE, config.COMMIT_FILE_MAX_SIZE / 1024 / 1024)
            )


class CommittedFileExtensionCheck(CommittedFileCheck):
    """Special check for committed file extension"""

    def get_problems(self):
        illegal_suffixes = config.BINARY_FILE_ILLEGAL_SUFFIXES
        illegal_suffixes_list = illegal_suffixes.split(",")
        legal_binary_filenames = config.LEGAL_BINARY_FILENAMES
        legal_binary_filenames_list = legal_binary_filenames.split(",")
        filename = self.committed_file.get_filename()
        if filename in legal_binary_filenames_list:
            return
        extension = self.committed_file.get_extension()
        if extension in illegal_suffixes_list:
            yield (
                Severity.ERROR,
                '提交 {} 的文件 {} 在不允许的提交文件后缀名清单中 {}'
                    .format(self.committed_file.commit, self.committed_file.path,
                            illegal_suffixes_list)
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""sc-githooks - Setup

Copyright (c) 2021 Scott Lau
Portions Copyright (c) 2021 InnoGames GmbH
Portions Copyright (c) 2021 Emre Hasegeli
"""

from setuptools import setup, find_packages

from githooks import VERSION
from githooks.config import checks

with open('README.rst') as fd:
    readme = fd.read()

setup(
    name='sc-githooks',
    version='.'.join(str(v) for v in VERSION),
    url='https://github.com/Scott-Lau/sc-githooks',
    packages=find_packages(),
    author='Scott Lau',
    author_email='exceedego@126.com',
    license='MIT',
    platforms='POSIX',
    description='Git pre-receive hook to check commits',
    long_description=readme,
    keywords=(
            'syntax-checker git git-hook python ' +
            ' '.join(c.args[0] for c in checks if hasattr(c, 'args'))
    ),
    entry_points={
        'console_scripts': [
            'sc-pre-receive=githooks.pre_receive:main',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
)



"""sc-githooks - The base check class

Copyright (c) 2021 Scott Lau
Portions Copyright (c) 2021 InnoGames GmbH
Portions Copyright (c) 2021 Emre Hasegeli
"""

from enum import IntEnum


class CheckState(IntEnum):
    NEW = 0
    CLONED = 1
    DONE = 2
    FAILED = 3


class Severity(IntEnum):
    # The numbers are selected to match the Syslog standard.
    ERROR = 3
    WARNING = 4
    NOTICE = 5
    NOTE = 5
    INFO = 6

    def translate(self):
        if self.__eq__(Severity.ERROR):
            return "错误"
        elif self.__eq__(Severity.WARNING):
            return "警告"
        elif self.__eq__(Severity.NOTICE):
            return "注意"
        elif self.__eq__(Severity.NOTE):
            return "注意"
        elif self.__eq__(Severity.INFO):
            return "信息"
        else:
            return "未知"

    @classmethod
    def split(cls, line):
        """Search the severities in the beginning of the string

        It returns the highest severity when non match.
        """
        for name, severity in cls._member_map_.items():
            if line.upper().startswith(name):
                line = line[len(name):].strip(' :-')
                break
        return severity, line


class BaseCheck:
    """The parent class of all checks

    Checks are expanded to different objects by cloning. The subclasses
    has to override prepare() method to clone the check at appropriate
    stage.
    """
    preferred_checks = []
    state = CheckState.NEW

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            # We expect all of the arguments to be initialized with defaults
            # on the class.
            assert hasattr(type(self), key)
            if value:
                setattr(self, key, value)

    def clone(self):
        new = type(self)(**vars(self))
        new.state = CheckState.CLONED
        return new

    def set_state(self, state):
        assert state > CheckState.CLONED
        self.state = max(self.state, state)

    def prepare(self, obj):
        for check in self.preferred_checks:
            if check.prepare(obj):
                return None
        return self

    def print_problems(self):
        header_printed = False
        for severity, problem in self.evaluate_problems():
            if not header_printed:
                print('=== {} ==='.format(self))
                header_printed = True
            print('{}: {}'.format(severity.translate(), problem))
        if header_printed:
            print('')
        self.set_state(CheckState.DONE)

    def evaluate_problems(self):
        assert self.state == CheckState.CLONED
        for severity, problem in self.get_problems():
            if severity <= Severity.ERROR:
                self.set_state(CheckState.FAILED)
            yield severity, problem

    def __str__(self):
        return type(self).__name__


def prepare_checks(checks, obj, next_checks=None):
    """Prepare the checks to the object

    It yields the checks prepared and ready.  The checks which are not
    ready yet are going do be appended to the next_checks list.
    """
    for check in checks:
        prepared_check = check.prepare(obj)
        if prepared_check:
            cloned = prepared_check.state >= CheckState.CLONED
            assert next_checks is not None or cloned

            if cloned:
                yield prepared_check
            else:
                next_checks.append(prepared_check)



"""sc-githooks - Configuration of the checks

Copyright (c) 2021 Scott Lau
Portions Copyright (c) 2021 InnoGames GmbH
Portions Copyright (c) 2021 Emre Hasegeli
"""

from githooks.commit_checks import (
    CheckCommitMessage,
    CheckCommitSummary,
    CheckChangedFilePaths,
    CheckBinaryFiles,
)
from githooks.commit_list_checks import (
    CheckDuplicateCommitSummaries,
    CheckMisleadingMergeCommit,
    CheckTimestamps,
    CheckContributors,
)
from githooks.file_checks import (
    CommittedFileSizeCheck,
    CommittedFileExtensionCheck,
)

# 是否开发模式
DEV_MODE = False
# DEV_MODE = True

checks = []  # type: list

# Commit list checks
# 检查多个提交的提交标题是否重复
checks.append(CheckDuplicateCommitSummaries())
# 检查合并提交
checks.append(CheckMisleadingMergeCommit())
# 检查时间标签信息
checks.append(CheckTimestamps())
# 检查提交者信息
checks.append(CheckContributors())

# 提交标题行最大长度
COMMIT_SUMMARY_MAX_LENGTH = 50
# 提交详细行最大行长度
COMMIT_LINE_MAX_LENGTH = 80

# Commit checks
# 提交消息（包含标题）检查
checks.append(CheckCommitMessage())
# 提交标题检查
checks.append(CheckCommitSummary())
# 文件路径检查
checks.append(CheckChangedFilePaths())

# 检查二进制文件
checks.append(CheckBinaryFiles())

# 二进制文件黑名单（逗号分隔的后缀名清单，无需带.，通过后缀名控制）
BINARY_FILE_ILLEGAL_SUFFIXES = "jar"
# 二进制文件白名单（逗号分隔的区分大小写的文件名列表，这些二进制文件允许提交）
LEGAL_BINARY_FILENAMES = "gradle-wrapper.jar,maven-wrapper.jar"
# 检查文件后缀名
checks.append(CommittedFileExtensionCheck())

# 提交文件最大大小(5MB)
COMMIT_FILE_MAX_SIZE = 5242880

# Commit file size check
checks.append(CommittedFileSizeCheck())
